@@ -14,22 +14,39 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFCx. If not, see <http://www.gnu.org/licenses/>.
-#
-# The bilinear form a(u, v) and Linear form L(v) for the Stokes
-# equations using a mixed formulation (equal-order stabilized).
-import basix.ufl
-from ufl import (Coefficient, TestFunctions, TrialFunctions, div, dot, dx,
-                 grad, inner)
+"""Stabilised Stokes demo.
 
-vector = basix.ufl.element("Lagrange", "triangle", 1, shape=(2, ))
+The bilinear form a(u, v) and Linear form L(v) for the Stokes
+equations using a mixed formulation (equal-order stabilized).
+"""
+
+import basix.ufl
+from ufl import (
+    Coefficient,
+    FunctionSpace,
+    Mesh,
+    TestFunctions,
+    TrialFunctions,
+    div,
+    dot,
+    dx,
+    grad,
+    inner,
+)
+
+vector = basix.ufl.element("Lagrange", "triangle", 1, shape=(2,))
 scalar = basix.ufl.element("Lagrange", "triangle", 1)
 system = basix.ufl.mixed_element([vector, scalar])
+domain = Mesh(basix.ufl.element("Lagrange", "triangle", 1, shape=(2,)))
+system_space = FunctionSpace(domain, system)
+scalar_space = FunctionSpace(domain, scalar)
+vector_space = FunctionSpace(domain, vector)
 
-(u, p) = TrialFunctions(system)
-(v, q) = TestFunctions(system)
+(u, p) = TrialFunctions(system_space)
+(v, q) = TestFunctions(system_space)
 
-f = Coefficient(vector)
-h = Coefficient(scalar)
+f = Coefficient(vector_space)
+h = Coefficient(scalar_space)
 
 beta = 0.2
 delta = beta * h * h
