@@ -14,17 +14,21 @@
 #
 # You should have received a copy of the GNU Lesser General Public License
 # along with FFCx. If not, see <http://www.gnu.org/licenses/>.
-#
-# The bilinear form a(u, v) and linear form L(v) for
-# the vector-valued Poisson's equation.
+"""Vector Poisson demo.
+
+The bilinear form a(u, v) and linear form L(v) for the vector-valued Poisson's equation.
+"""
+
 import basix.ufl
-from ufl import Coefficient, TestFunction, TrialFunction, dx, grad, inner
+from ufl import Coefficient, FunctionSpace, Mesh, TestFunction, TrialFunction, dx, grad, inner
 
-element = basix.ufl.element("Lagrange", "triangle", 1, shape=(2, ))
+element = basix.ufl.element("Lagrange", "triangle", 1, shape=(2,))
+domain = Mesh(element)
+space = FunctionSpace(domain, element)
 
-u = TrialFunction(element)
-v = TestFunction(element)
-f = Coefficient(element)
+u = TrialFunction(space)
+v = TestFunction(space)
+f = Coefficient(space)
 
 a = inner(grad(u), grad(v)) * dx
 L = inner(f, v) * dx
