@@ -10,22 +10,25 @@
 # old implementation in FFC
 """Generate UFC code for a form."""
 
+from __future__ import annotations
+
 import logging
 
 import numpy as np
 
 from ffcx.codegeneration.C import form_template
+from ffcx.ir.representation import FormIR
 
 logger = logging.getLogger("ffcx")
 
 
-def generator(ir, options):
+def generator(ir: FormIR, options):
     """Generate UFC code for a form."""
     logger.info("Generating code for form:")
     logger.info(f"--- rank: {ir.rank}")
     logger.info(f"--- name: {ir.name}")
 
-    d = {}
+    d: dict[str, int | str] = {}
     d["factory_name"] = ir.name
     d["name_from_uflfile"] = ir.name_from_uflfile
     d["signature"] = f'"{ir.signature}"'
@@ -33,17 +36,17 @@ def generator(ir, options):
     d["num_coefficients"] = ir.num_coefficients
     d["num_constants"] = ir.num_constants
 
-    if len(ir.original_coefficient_position) > 0:
-        values = ", ".join(str(i) for i in ir.original_coefficient_position)
-        sizes = len(ir.original_coefficient_position)
+    if len(ir.original_coefficient_positions) > 0:
+        values = ", ".join(str(i) for i in ir.original_coefficient_positions)
+        sizes = len(ir.original_coefficient_positions)
 
         d["original_coefficient_position_init"] = (
             f"int original_coefficient_position_{ir.name}[{sizes}] = {{{values}}};"
         )
-        d["original_coefficient_position"] = f"original_coefficient_position_{ir.name}"
+        d["original_coefficient_positions"] = f"original_coefficient_position_{ir.name}"
     else:
         d["original_coefficient_position_init"] = ""
-        d["original_coefficient_position"] = "NULL"
+        d["original_coefficient_positions"] = "NULL"
 
     if len(ir.coefficient_names) > 0:
         values = ", ".join(f'"{name}"' for name in ir.coefficient_names)
@@ -67,25 +70,18 @@ def generator(ir, options):
         d["constant_names_init"] = ""
         d["constant_names"] = "NULL"
 
-    if len(ir.finite_elements) > 0:
-        d["finite_elements"] = f"finite_elements_{ir.name}"
-        values = ", ".join(f"&{el}" for el in ir.finite_elements)
-        sizes = len(ir.finite_elements)
-        d["finite_elements_init"] = (
-            f"ufcx_finite_element* finite_elements_{ir.name}[{sizes}] = {{{values}}};"
+    if len(ir.finite_element_hashes) > 0:
+        d["finite_element_hashes"] = f"finite_element_hashes_{ir.name}"
+        values = ", ".join(
+            f"UINT64_C({0 if el is None else el})" for el in ir.finite_element_hashes
+        )
+        sizes = len(ir.finite_element_hashes)
+        d["finite_element_hashes_init"] = (
+            f"uint64_t finite_element_hashes_{ir.name}[{sizes}] = {{{values}}};"
         )
     else:
-        d["finite_elements"] = "NULL"
-        d["finite_elements_init"] = ""
-
-    if len(ir.dofmaps) > 0:
-        d["dofmaps"] = f"dofmaps_{ir.name}"
-        values = ", ".join(f"&{dofmap}" for dofmap in ir.dofmaps)
-        sizes = len(ir.dofmaps)
-        d["dofmaps_init"] = f"ufcx_dofmap* dofmaps_{ir.name}[{sizes}] = {{{values}}};"
-    else:
-        d["dofmaps"] = "NULL"
-        d["dofmaps_init"] = ""
+        d["finite_element_hashes"] = "NULL"
+        d["finite_element_hashes_init"] = ""
 
     integrals = []
     integral_ids = []
@@ -127,48 +123,6 @@ def generator(ir, options):
         f"int form_integral_offsets_{ir.name}[{sizes}] = {{{values}}};"
     )
 
-    vs_code = []
-    code = []
-
-    # FIXME: Should be handled differently, revise how
-    # ufcx_function_space is generated
-    for name, (
-        element,
-        dofmap,
-        cmap_family,
-        cmap_degree,
-        cmap_celltype,
-        cmap_variant,
-        value_shape,
-    ) in ir.function_spaces.items():
-        code += [f"static ufcx_function_space functionspace_{name} ="]
-        code += ["{"]
-        code += [f".finite_element = &{element},"]
-        code += [f".dofmap = &{dofmap},"]
-        code += [f'.geometry_family = "{cmap_family}",']
-        code += [f".geometry_degree = {cmap_degree},"]
-        code += [f".geometry_basix_cell = {int(cmap_celltype)},"]
-        code += [f".geometry_basix_variant = {int(cmap_variant)},"]
-        code += [f".value_rank = {len(value_shape)},"]
-        if len(value_shape) == 0:
-            code += [".value_shape = NULL"]
-        else:
-            vs_code += [
-                f"int value_shape_{ir.name}_{name}[{len(value_shape)}] = {{",
-                "  " + ", ".join([f"{i}" for i in value_shape]),
-                "};",
-            ]
-            code += [f".value_shape = value_shape_{ir.name}_{name}"]
-        code += ["};"]
-
-    for name in ir.function_spaces.keys():
-        code += [f'if (strcmp(function_name, "{name}") == 0) return &functionspace_{name};']
-
-    code += ["return NULL;\n"]
-
-    d["functionspace"] = "\n".join(code)
-    d["value_shape_init"] = "\n".join(vs_code)
-
     # Check that no keys are redundant or have been missed
     from string import Formatter
 
