@@ -6,6 +6,7 @@
 """Generate UFC code for an integral."""
 
 import logging
+import sys
 
 import numpy as np
 
@@ -22,11 +23,11 @@ logger = logging.getLogger("ffcx")
 def generator(ir: IntegralIR, options):
     """Generate C code for an integral."""
     logger.info("Generating code for integral:")
-    logger.info(f"--- type: {ir.integral_type}")
-    logger.info(f"--- name: {ir.name}")
+    logger.info(f"--- type: {ir.expression.integral_type}")
+    logger.info(f"--- name: {ir.expression.name}")
 
     """Generate code for an integral."""
-    factory_name = ir.name
+    factory_name = ir.expression.name
 
     # Format declaration
     declaration = ufcx_integrals.declaration.format(factory_name=factory_name)
@@ -51,31 +52,39 @@ def generator(ir: IntegralIR, options):
         values = ", ".join("1" if i else "0" for i in ir.enabled_coefficients)
         sizes = len(ir.enabled_coefficients)
         code["enabled_coefficients_init"] = (
-            f"bool enabled_coefficients_{ir.name}[{sizes}] = {{{values}}};"
+            f"bool enabled_coefficients_{ir.expression.name}[{sizes}] = {{{values}}};"
         )
-        code["enabled_coefficients"] = f"enabled_coefficients_{ir.name}"
+        code["enabled_coefficients"] = f"enabled_coefficients_{ir.expression.name}"
     else:
         code["enabled_coefficients_init"] = ""
         code["enabled_coefficients"] = "NULL"
 
     code["tabulate_tensor"] = body
 
-    code["tabulate_tensor_float32"] = "NULL"
-    code["tabulate_tensor_float64"] = "NULL"
-    code["tabulate_tensor_complex64"] = "NULL"
-    code["tabulate_tensor_complex128"] = "NULL"
+    code["tabulate_tensor_float32"] = ".tabulate_tensor_float32 = NULL,"
+    code["tabulate_tensor_float64"] = ".tabulate_tensor_float64 = NULL,"
+    if sys.platform.startswith("win32"):
+        code["tabulate_tensor_complex64"] = ""
+        code["tabulate_tensor_complex128"] = ""
+    else:
+        code["tabulate_tensor_complex64"] = ".tabulate_tensor_complex64 = NULL,"
+        code["tabulate_tensor_complex128"] = ".tabulate_tensor_complex128 = NULL,"
     np_scalar_type = np.dtype(options["scalar_type"]).name
-    code[f"tabulate_tensor_{np_scalar_type}"] = f"tabulate_tensor_{factory_name}"
+    code[f"tabulate_tensor_{np_scalar_type}"] = (
+        f".tabulate_tensor_{np_scalar_type} = tabulate_tensor_{factory_name},"
+    )
+
+    element_hash = 0 if ir.coordinate_element_hash is None else ir.coordinate_element_hash
 
     implementation = ufcx_integrals.factory.format(
         factory_name=factory_name,
         enabled_coefficients=code["enabled_coefficients"],
         enabled_coefficients_init=code["enabled_coefficients_init"],
         tabulate_tensor=code["tabulate_tensor"],
-        needs_facet_permutations="true" if ir.needs_facet_permutations else "false",
+        needs_facet_permutations="true" if ir.expression.needs_facet_permutations else "false",
         scalar_type=dtype_to_c_type(options["scalar_type"]),
         geom_type=dtype_to_c_type(dtype_to_scalar_dtype(options["scalar_type"])),
-        coordinate_element=f"&{ir.coordinate_element}",
+        coordinate_element_hash=f"UINT64_C({element_hash})",
         tabulate_tensor_float32=code["tabulate_tensor_float32"],
         tabulate_tensor_float64=code["tabulate_tensor_float64"],
         tabulate_tensor_complex64=code["tabulate_tensor_complex64"],
