@@ -26,12 +26,12 @@ void tabulate_tensor_{factory_name}({scalar_type}* restrict A,
 ufcx_integral {factory_name} =
 {{
   .enabled_coefficients = {enabled_coefficients},
-  .tabulate_tensor_float32 = {tabulate_tensor_float32},
-  .tabulate_tensor_float64 = {tabulate_tensor_float64},
-  .tabulate_tensor_complex64 = {tabulate_tensor_complex64},
-  .tabulate_tensor_complex128 = {tabulate_tensor_complex128},
+  {tabulate_tensor_float32}
+  {tabulate_tensor_float64}
+  {tabulate_tensor_complex64}
+  {tabulate_tensor_complex128}
   .needs_facet_permutations = {needs_facet_permutations},
-  .coordinate_element = {coordinate_element},
+  .coordinate_element_hash = {coordinate_element_hash},
 }};
 
 // End of code for integral {factory_name}
