@@ -6,9 +6,8 @@
 # SPDX-License-Identifier:    LGPL-3.0-or-later
 """Compiler stage 2: Code representation.
 
-Module computes intermediate representations of forms, elements and
-dofmaps. For each UFC function, we extract the data needed for code
-generation at a later stage.
+Module computes intermediate representations of forms. For each UFC
+function, we extract the data needed for code generation at a later stage.
 
 The representation should conform strictly to the naming and order of
 functions in UFC. Thus, for code generation of the function "foo", one
@@ -23,8 +22,6 @@ import logging
 import typing
 import warnings
 
-import basix
-import basix.ufl
 import numpy as np
 import numpy.typing as npt
 import ufl
@@ -49,35 +46,14 @@ class FormIR(typing.NamedTuple):
     num_coefficients: int
     num_constants: int
     name_from_uflfile: str
-    function_spaces: dict[
-        str, tuple[str, str, str, int, basix.CellType, basix.LagrangeVariant, tuple[int]]
-    ]
-    original_coefficient_position: list[int]
+    original_coefficient_positions: list[int]
     coefficient_names: list[str]
     constant_names: list[str]
-    finite_elements: list[str]
-    dofmaps: list[str]
+    finite_element_hashes: list[int]
     integral_names: dict[str, list[str]]
     subdomain_ids: dict[str, list[int]]
 
 
-class CustomElementIR(typing.NamedTuple):
-    """Intermediate representation of a custom element."""
-
-    cell_type: basix.CellType
-    value_shape: tuple[int, ...]
-    wcoeffs: npt.NDArray[np.float64]
-    x: list[list[npt.NDArray[np.float64]]]
-    M: list[list[npt.NDArray[np.float64]]]
-    map_type: basix.MapType
-    sobolev_space: basix.SobolevSpace
-    interpolation_nderivs: int
-    discontinuous: bool
-    embedded_subdegree: int
-    embedded_superdegree: int
-    polyset_type: basix.PolysetType
-
-
 class QuadratureIR(typing.NamedTuple):
     """Intermediate representation of a quadrature rule."""
 
@@ -86,98 +62,45 @@ class QuadratureIR(typing.NamedTuple):
     weights: npt.NDArray[np.float64]
 
 
-class ElementIR(typing.NamedTuple):
-    """Intermediate representation of an element."""
+class CommonExpressionIR(typing.NamedTuple):
+    """Common-ground for IntegralIR and ExpressionIR."""
 
-    id: int
+    integral_type: str
+    entity_type: str
+    tensor_shape: list[int]
+    coefficient_numbering: dict[ufl.Coefficient, int]
+    coefficient_offsets: dict[ufl.Coefficient, int]
+    original_constant_offsets: dict[ufl.Constant, int]
+    unique_tables: dict[str, npt.NDArray[np.float64]]
+    unique_table_types: dict[str, str]
+    integrand: dict[QuadratureRule, dict]
     name: str
-    signature: str
-    cell_shape: str
-    topological_dimension: int
-    space_dimension: int
-    reference_value_shape: tuple[int, ...]
-    degree: int
-    symmetric: bool
-    num_sub_elements: int
-    block_size: int
-    sub_elements: list[str]
-    element_type: str
-    entity_dofs: list[list[list[int]]]
-    lagrange_variant: basix.LagrangeVariant
-    dpc_variant: basix.DPCVariant
-    basix_family: basix.ElementFamily
-    basix_cell: basix.CellType
-    discontinuous: bool
-    custom_element: CustomElementIR
-    custom_quadrature: QuadratureIR
-
-
-class DofMapIR(typing.NamedTuple):
-    """Intermediate representation of a DOF map."""
-
-    id: int
-    name: str
-    signature: str
-    num_global_support_dofs: int
-    num_element_support_dofs: int
-    entity_dofs: list[list[list[int]]]
-    entity_closure_dofs: list[list[list[int]]]
-    num_entity_closure_dofs: list[list[int]]
-    num_sub_dofmaps: int
-    sub_dofmaps: list[str]
-    block_size: int
+    needs_facet_permutations: bool
+    shape: list[int]
 
 
 class IntegralIR(typing.NamedTuple):
     """Intermediate representation of an integral."""
 
-    integral_type: str
+    expression: CommonExpressionIR
     rank: int
-    entitytype: str
     enabled_coefficients: list[bool]
-    tensor_shape: list[int]
-    coefficient_numbering: dict[ufl.Coefficient, int]
-    coefficient_offsets: dict[ufl.Coefficient, int]
-    original_constant_offsets: dict[ufl.Constant, int]
-    unique_tables: dict[str, npt.NDArray[np.float64]]
-    unique_table_types: dict[str, str]
-    integrand: dict[QuadratureRule, dict]
-    name: str
-    needs_facet_permutations: bool
-    coordinate_element: str
+    coordinate_element_hash: str
 
 
 class ExpressionIR(typing.NamedTuple):
-    """Intermediate representation of an expression."""
+    """Intermediate representation of a DOLFINx Expression."""
 
-    name: str
-    options: dict
-    unique_tables: dict[str, npt.NDArray[np.float64]]
-    unique_table_types: dict[str, str]
-    integrand: dict[QuadratureRule, dict]
-    coefficient_numbering: dict[ufl.Coefficient, int]
-    coefficient_offsets: dict[ufl.Coefficient, int]
-    integral_type: str
-    entitytype: str
-    tensor_shape: list[int]
-    expression_shape: list[int]
-    original_constant_offsets: dict[ufl.Constant, int]
-    points: npt.NDArray[np.float64]
+    expression: CommonExpressionIR
+    original_coefficient_positions: list[int]
     coefficient_names: list[str]
     constant_names: list[str]
-    needs_facet_permutations: bool
-    function_spaces: dict[
-        str, tuple[str, str, str, int, basix.CellType, basix.LagrangeVariant, tuple[int]]
-    ]
     name_from_uflfile: str
-    original_coefficient_positions: list[int]
 
 
 class DataIR(typing.NamedTuple):
     """Intermediate representation of data."""
 
-    elements: list[ElementIR]
-    dofmaps: list[DofMapIR]
     integrals: list[IntegralIR]
     forms: list[FormIR]
     expressions: list[ExpressionIR]
@@ -198,10 +121,7 @@ def compute_ir(
     # Compute object names
     # NOTE: This is done here for performance reasons, because repeated calls
     # within each IR computation would be expensive due to UFL signature computations
-    finite_element_names = {
-        e: naming.finite_element_name(e, prefix) for e in analysis.unique_elements
-    }
-    dofmap_names = {e: naming.dofmap_name(e, prefix) for e in analysis.unique_elements}
+    finite_element_hashes = {e: e.basix_hash() for e in analysis.unique_elements}
     integral_names = {}
     form_names = {}
     for fd_index, fd in enumerate(analysis.form_data):
@@ -211,23 +131,13 @@ def compute_ir(
                 fd.original_form, itg_data.integral_type, fd_index, itg_data.subdomain_id, prefix
             )
 
-    ir_elements = [
-        _compute_element_ir(e, analysis.element_numbers, finite_element_names)
-        for e in analysis.unique_elements
-    ]
-
-    ir_dofmaps = [
-        _compute_dofmap_ir(e, analysis.element_numbers, dofmap_names)
-        for e in analysis.unique_elements
-    ]
-
     irs = [
         _compute_integral_ir(
             fd,
             i,
             analysis.element_numbers,
             integral_names,
-            finite_element_names,
+            finite_element_hashes,
             options,
             visualise,
         )
@@ -243,8 +153,7 @@ def compute_ir(
             form_names,
             integral_names,
             analysis.element_numbers,
-            finite_element_names,
-            dofmap_names,
+            finite_element_hashes,
             object_names,
         )
         for (i, fd) in enumerate(analysis.form_data)
@@ -259,133 +168,26 @@ def compute_ir(
             options,
             visualise,
             object_names,
-            finite_element_names,
-            dofmap_names,
+            finite_element_hashes,
         )
         for i, expr in enumerate(analysis.expressions)
     ]
 
     return DataIR(
-        elements=ir_elements,
-        dofmaps=ir_dofmaps,
         integrals=ir_integrals,
         forms=ir_forms,
         expressions=ir_expressions,
     )
 
 
-def _compute_element_ir(element, element_numbers, finite_element_names):
-    """Compute intermediate representation of element."""
-    logger.info(f"Computing IR for element {element}")
-
-    # Create basix elements
-    cell = element.cell
-
-    # Store id
-    ir = {"id": element_numbers[element]}
-    ir["name"] = finite_element_names[element]
-
-    # Compute data for each function
-    ir["signature"] = repr(element)
-    ir["cell_shape"] = element.cell_type.name
-    ir["topological_dimension"] = cell.topological_dimension()
-    ir["space_dimension"] = element.dim + element.num_global_support_dofs
-    ir["element_type"] = element.ufcx_element_type
-    ir["lagrange_variant"] = element.lagrange_variant
-    ir["dpc_variant"] = element.dpc_variant
-    ir["basix_family"] = element.element_family
-    ir["basix_cell"] = element.cell_type
-    ir["discontinuous"] = element.discontinuous
-    ir["degree"] = element.degree
-    ir["symmetric"] = isinstance(element, basix.ufl._BlockedElement) and isinstance(
-        element._pullback, ufl.pullback.SymmetricPullback
-    )
-    ir["reference_value_shape"] = element.reference_value_shape
-
-    ir["num_sub_elements"] = element.num_sub_elements
-    ir["sub_elements"] = [finite_element_names[e] for e in element.sub_elements]
-
-    ir["block_size"] = element.block_size
-    if element.block_size > 1:
-        element = element._sub_element
-
-    ir["entity_dofs"] = element.entity_dofs
-
-    if element.is_custom_element:
-        ir["custom_element"] = _compute_custom_element_ir(element._element)
-    else:
-        ir["custom_element"] = None
-
-    if element.has_custom_quadrature:
-        ir["custom_quadrature"] = _compute_custom_quadrature_ir(element)
-    else:
-        ir["custom_quadrature"] = None
-
-    return ElementIR(**ir)
-
-
-def _compute_custom_element_ir(
-    basix_element: basix.finite_element.FiniteElement,
-) -> CustomElementIR:
-    """Compute intermediate representation of a custom Basix element."""
-    ir: dict[str, typing.Any] = {}
-    ir["cell_type"] = basix_element.cell_type
-    ir["value_shape"] = basix_element.value_shape
-    ir["wcoeffs"] = basix_element.wcoeffs
-    ir["x"] = basix_element.x
-    ir["M"] = basix_element.M
-    ir["map_type"] = basix_element.map_type
-    ir["sobolev_space"] = basix_element.sobolev_space
-    ir["discontinuous"] = basix_element.discontinuous
-    ir["interpolation_nderivs"] = basix_element.interpolation_nderivs
-    ir["embedded_subdegree"] = basix_element.embedded_subdegree
-    ir["embedded_superdegree"] = basix_element.embedded_superdegree
-    ir["polyset_type"] = basix_element.polyset_type
-
-    return CustomElementIR(**ir)
-
-
-def _compute_custom_quadrature_ir(element: basix.ufl._ElementBase) -> QuadratureIR:
-    """Compute intermediate representation of a custom Basix element."""
-    ir: dict[str, typing.Any] = {}
-    ir["cell_shape"] = element.cell_type.name
-    ir["points"], ir["weights"] = element.custom_quadrature()
-
-    return QuadratureIR(**ir)
-
-
-def _compute_dofmap_ir(element, element_numbers, dofmap_names):
-    """Compute intermediate representation of dofmap."""
-    logger.info(f"Computing IR for dofmap of {element}")
-
-    # Store id
-    ir = {"id": element_numbers[element]}
-    ir["name"] = dofmap_names[element]
-
-    # Compute data for each function
-    ir["signature"] = "FFCx dofmap for " + repr(element)
-    ir["sub_dofmaps"] = [dofmap_names[e] for e in element.sub_elements]
-    ir["num_sub_dofmaps"] = element.num_sub_elements
-
-    ir["block_size"] = element.block_size
-    if element.block_size > 1:
-        element = element._sub_element
-
-    ir["entity_dofs"] = element.entity_dofs
-    ir["entity_closure_dofs"] = element.entity_closure_dofs
-
-    num_dofs_per_entity_closure = [i[0] for i in element.num_entity_closure_dofs]
-    ir["num_entity_closure_dofs"] = num_dofs_per_entity_closure
-    ir["entity_closure_dofs"] = element.entity_closure_dofs
-
-    ir["num_global_support_dofs"] = element.num_global_support_dofs
-    ir["num_element_support_dofs"] = element.dim
-
-    return DofMapIR(**ir)
-
-
 def _compute_integral_ir(
-    form_data, form_index, element_numbers, integral_names, finite_element_names, options, visualise
+    form_data,
+    form_index,
+    element_numbers,
+    integral_names,
+    finite_element_hashes,
+    options,
+    visualise,
 ) -> list[IntegralIR]:
     """Compute intermediate representation for form integrals."""
     _entity_types = {
@@ -400,20 +202,26 @@ def _compute_integral_ir(
     irs = []
     for itg_data_index, itg_data in enumerate(form_data.integral_data):
         logger.info(f"Computing IR for integral in integral group {itg_data_index}")
+        expression_ir = {}
 
         # Compute representation
-        entitytype = _entity_types[itg_data.integral_type]
+        entity_type = _entity_types[itg_data.integral_type]
         cell = itg_data.domain.ufl_cell()
         cellname = cell.cellname()
         tdim = cell.topological_dimension()
         assert all(tdim == itg.ufl_domain().topological_dimension() for itg in itg_data.integrals)
 
-        ir = {
+        expression_ir = {
             "integral_type": itg_data.integral_type,
+            "entity_type": entity_type,
+            "shape": (),
+        }
+        ir = {
             "rank": form_data.rank,
-            "entitytype": entitytype,
             "enabled_coefficients": itg_data.enabled_coefficients,
-            "coordinate_element": finite_element_names[itg_data.domain.ufl_coordinate_element()],
+            "coordinate_element_hash": finite_element_hashes[
+                itg_data.domain.ufl_coordinate_element()
+            ],
         }
 
         # Get element space dimensions
@@ -429,10 +237,10 @@ def _compute_integral_ir(
         ]
 
         # Compute shape of element tensor
-        if ir["integral_type"] == "interior_facet":
-            ir["tensor_shape"] = [2 * dim for dim in argument_dimensions]
+        if expression_ir["integral_type"] == "interior_facet":
+            expression_ir["tensor_shape"] = [2 * dim for dim in argument_dimensions]
         else:
-            ir["tensor_shape"] = argument_dimensions
+            expression_ir["tensor_shape"] = argument_dimensions
 
         integral_type = itg_data.integral_type
         cell = itg_data.domain.ufl_cell()
@@ -557,7 +365,7 @@ def _compute_integral_ir(
             coefficient_numbering[f] = i
 
         # Add coefficient numbering to IR
-        ir["coefficient_numbering"] = coefficient_numbering
+        expression_ir["coefficient_numbering"] = coefficient_numbering
 
         index_to_coeff = sorted([(v, k) for k, v in coefficient_numbering.items()])
         offsets = {}
@@ -568,7 +376,7 @@ def _compute_integral_ir(
             _offset += width * element_dimensions[el]
 
         # Copy offsets also into IR
-        ir["coefficient_offsets"] = offsets
+        expression_ir["coefficient_offsets"] = offsets
 
         # Build offsets for Constants
         original_constant_offsets = {}
@@ -577,7 +385,7 @@ def _compute_integral_ir(
             original_constant_offsets[constant] = _offset
             _offset += np.prod(constant.ufl_shape, dtype=int)
 
-        ir["original_constant_offsets"] = original_constant_offsets
+        expression_ir["original_constant_offsets"] = original_constant_offsets
 
         # Create map from number of quadrature points -> integrand
         integrand_map: dict[QuadratureRule, ufl.core.expr.Expr] = {
@@ -588,18 +396,18 @@ def _compute_integral_ir(
         integral_ir = compute_integral_ir(
             itg_data.domain.ufl_cell(),
             itg_data.integral_type,
-            ir["entitytype"],
+            expression_ir["entity_type"],
             integrand_map,
-            ir["tensor_shape"],
+            expression_ir["tensor_shape"],
             options,
             visualise,
         )
 
-        ir.update(integral_ir)
+        expression_ir.update(integral_ir)
 
         # Fetch name
-        ir["name"] = integral_names[(form_index, itg_data_index)]
-
+        expression_ir["name"] = integral_names[(form_index, itg_data_index)]
+        ir["expression"] = CommonExpressionIR(**expression_ir)
         irs.append(IntegralIR(**ir))
 
     return irs
@@ -612,8 +420,7 @@ def _compute_form_ir(
     form_names,
     integral_names,
     element_numbers,
-    finite_element_names,
-    dofmap_names,
+    finite_element_hashes,
     object_names,
 ) -> FormIR:
     """Compute intermediate representation of form."""
@@ -640,45 +447,15 @@ def _compute_form_ir(
         for j, obj in enumerate(form_data.original_form.constants())
     ]
 
-    ir["original_coefficient_position"] = form_data.original_coefficient_positions
+    ir["original_coefficient_positions"] = form_data.original_coefficient_positions
 
-    ir["finite_elements"] = [
-        finite_element_names[e]
+    ir["finite_element_hashes"] = [
+        finite_element_hashes[e]
         for e in form_data.argument_elements + form_data.coefficient_elements
     ]
 
-    ir["dofmaps"] = [
-        dofmap_names[e] for e in form_data.argument_elements + form_data.coefficient_elements
-    ]
-
-    fs = {}
-    for function in form_data.original_form.arguments() + tuple(form_data.reduced_coefficients):
-        name = object_names.get(id(function), str(function))
-        if not str(name).isidentifier():
-            raise ValueError(f'Function name "{name}" must be a valid object identifier.')
-        el = function.ufl_function_space().ufl_element()
-        space = function.ufl_function_space()
-        domain = space.ufl_domain()
-        cmap = domain.ufl_coordinate_element()
-        # Default point spacing for CoordinateElement is equispaced
-        if not isinstance(cmap, basix.ufl._ElementBase) and cmap.variant() is None:
-            cmap._sub_element._variant = "equispaced"
-        family = cmap.family_name
-        degree = cmap.degree
-        value_shape = space.value_shape
-        fs[name] = (
-            finite_element_names[el],
-            dofmap_names[el],
-            family,
-            degree,
-            cmap.cell_type,
-            cmap.lagrange_variant,
-            value_shape,
-        )
-
     form_name = object_names.get(id(form_data.original_form), form_id)
 
-    ir["function_spaces"] = fs
     ir["name_from_uflfile"] = f"form_{prefix}_{form_name}"
 
     # Store names of integrals and subdomain_ids for this form, grouped
@@ -712,18 +489,17 @@ def _compute_expression_ir(
     options,
     visualise,
     object_names,
-    finite_element_names,
-    dofmap_names,
+    finite_element_hashes,
 ):
     """Compute intermediate representation of expression."""
     logger.info(f"Computing IR for expression {index}")
 
     # Compute representation
     ir = {}
-
+    base_ir = {}
     original_expression = (expression[2], expression[1])
 
-    ir["name"] = naming.expression_name(original_expression, prefix)
+    base_ir["name"] = naming.expression_name(original_expression, prefix)
 
     original_expression = expression[2]
     points = expression[1]
@@ -749,9 +525,9 @@ def _compute_expression_ir(
     argument_dimensions = [element_dimensions[element] for element in argument_elements]
 
     tensor_shape = argument_dimensions
-    ir["tensor_shape"] = tensor_shape
+    base_ir["tensor_shape"] = tensor_shape
 
-    ir["expression_shape"] = list(expression.ufl_shape)
+    base_ir["shape"] = list(expression.ufl_shape)
 
     coefficients = ufl.algorithms.extract_coefficients(expression)
     coefficient_numbering = {}
@@ -759,7 +535,7 @@ def _compute_expression_ir(
         coefficient_numbering[coeff] = i
 
     # Add coefficient numbering to IR
-    ir["coefficient_numbering"] = coefficient_numbering
+    base_ir["coefficient_numbering"] = coefficient_numbering
 
     original_coefficient_positions = []
     original_coefficients = ufl.algorithms.extract_coefficients(original_expression)
@@ -775,31 +551,8 @@ def _compute_expression_ir(
         for j, obj in enumerate(ufl.algorithms.analysis.extract_constants(expression))
     ]
 
-    fs = {}
-    for function in tuple(original_coefficients) + tuple(arguments):
-        name = object_names.get(id(function), str(function))
-        if not str(name).isidentifier():
-            raise ValueError(f'Function name "{name}" must be a valid object identifier.')
-        el = function.ufl_function_space().ufl_element()
-        space = function.ufl_function_space()
-        domain = space.ufl_domain()
-        cmap = domain.ufl_coordinate_element()
-        family = cmap.family_name
-        degree = cmap.degree
-        value_shape = space.value_shape
-        fs[name] = (
-            finite_element_names[el],
-            dofmap_names[el],
-            family,
-            degree,
-            cmap.cell_type,
-            cmap.lagrange_variant,
-            value_shape,
-        )
-
     expression_name = object_names.get(id(original_expression), index)
 
-    ir["function_spaces"] = fs
     ir["name_from_uflfile"] = f"expression_{prefix}_{expression_name}"
 
     if len(argument_elements) > 1:
@@ -816,14 +569,14 @@ def _compute_expression_ir(
         _offset += element_dimensions[el]
 
     # Copy offsets also into IR
-    ir["coefficient_offsets"] = offsets
+    base_ir["coefficient_offsets"] = offsets
 
-    ir["integral_type"] = "expression"
+    base_ir["integral_type"] = "expression"
     if cell is not None:
         if (tdim := cell.topological_dimension()) == (pdim := points.shape[1]):
-            ir["entitytype"] = "cell"
+            base_ir["entity_type"] = "cell"
         elif tdim - 1 == pdim:
-            ir["entitytype"] = "facet"
+            base_ir["entity_type"] = "facet"
         else:
             raise ValueError(
                 f"Expression on domain with topological dimension {tdim}"
@@ -831,7 +584,7 @@ def _compute_expression_ir(
             )
     else:
         # For spatially invariant expressions, all expressions are evaluated in the cell
-        ir["entitytype"] = "cell"
+        base_ir["entity_type"] = "cell"
 
     # Build offsets for Constants
     original_constant_offsets = {}
@@ -840,9 +593,7 @@ def _compute_expression_ir(
         original_constant_offsets[constant] = _offset
         _offset += np.prod(constant.ufl_shape, dtype=int)
 
-    ir["original_constant_offsets"] = original_constant_offsets
-
-    ir["points"] = points
+    base_ir["original_constant_offsets"] = original_constant_offsets
 
     weights = np.array([1.0] * points.shape[0])
     rule = QuadratureRule(points, weights)
@@ -851,13 +602,19 @@ def _compute_expression_ir(
     if cell is None:
         assert (
             len(ir["original_coefficient_positions"]) == 0
-            and len(ir["original_constant_offsets"]) == 0
+            and len(base_ir["original_constant_offsets"]) == 0
         )
 
     expression_ir = compute_integral_ir(
-        cell, ir["integral_type"], ir["entitytype"], integrands, tensor_shape, options, visualise
+        cell,
+        base_ir["integral_type"],
+        base_ir["entity_type"],
+        integrands,
+        tensor_shape,
+        options,
+        visualise,
     )
-    ir["options"] = options
-    ir.update(expression_ir)
 
+    base_ir.update(expression_ir)
+    ir["expression"] = CommonExpressionIR(**base_ir)
     return ExpressionIR(**ir)
