@@ -148,7 +148,6 @@ def test_rank1(compile_args):
     )
 
     u_correct = np.array([f[1], f[0]]) + gradf0
-
     assert np.allclose(u_ffcx, u_correct.T)
 
 
@@ -166,7 +165,7 @@ def test_elimiate_zero_tables_tensor(compile_args):
     # Get vertices of cell
     # Coords storage XYZXYZXYZ
     basix_c_e = basix.create_element(
-        basix.ElementFamily.P, basix.cell.string_to_type(cell), 1, discontinuous=False
+        basix.ElementFamily.P, basix.CellType[cell], 1, discontinuous=False
     )
     coords = basix_c_e.points
 
@@ -174,9 +173,7 @@ def test_elimiate_zero_tables_tensor(compile_args):
     coeff_points = basix_c_e.points
 
     # Compile expression at interpolation points of second order Lagrange space
-    b_el = basix.create_element(
-        basix.ElementFamily.P, basix.cell.string_to_type(cell), 0, discontinuous=True
-    )
+    b_el = basix.create_element(basix.ElementFamily.P, basix.CellType[cell], 0, discontinuous=True)
     points = b_el.points
     obj, module, code = ffcx.codegeneration.jit.compile_expressions(
         [(expr, points)], cffi_extra_compile_args=compile_args
